'use strict';

exports.__esModule = true;

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _stcHelper = require('stc-helper');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * concurrent task dynamic limit
 */

var ConcurrentTaskLimit = function () {
  /**
   * contructor
   */

  function ConcurrentTaskLimit() {
    var initLimit = arguments.length <= 0 || arguments[0] === undefined ? 5 : arguments[0];
    var ignoreErrorFn = arguments[1];
    (0, _classCallCheck3.default)(this, ConcurrentTaskLimit);

    if (typeof initLimit === 'function') {
      ignoreErrorFn = initLimit;
      initLimit = 5;
    }
    this.limit = initLimit;
    this.ignoreErrorFn = ignoreErrorFn;
    this.doing = 0;
    this.index = 0;
    this.deferreds = [];
    this.hasDecrement = false;
  }
  /**
   * is ignore error
   */


  ConcurrentTaskLimit.prototype.isIgnoreError = function isIgnoreError(err) {
    if (!this.ignoreErrorFn) {
      return false;
    }
    return this.ignoreErrorFn(err);
  };
  /**
   * next 
   */


  ConcurrentTaskLimit.prototype.next = function next(flag, item) {
    if (flag === false) {
      this.limit--;
    } else if (flag === true && !this.hasDecrement) {
      this.limit++;
    }
    this.doing--;
    if (flag === false) {
      this.deferreds.push(item);
      this.hasDecrement = true;
    }
    if (flag) {
      this._runTask();
    }
    this._runTask();
  };
  /**
   * run task
   */


  ConcurrentTaskLimit.prototype._runTask = function _runTask() {
    var _this = this;

    if (this.doing >= this.limit || this.index >= this.deferreds.length) {
      if (this.doing === 0) {
        this.index = 0;
        this.deferreds = [];
      }
      return;
    }
    this.doing++;
    var item = this.deferreds[this.index++];
    var result = _promise2.default.resolve(item.fn());
    return result.then(function (data) {
      item.deferred.resolve(data);
      _this.next(true);
    }).catch(function (err) {
      if (_this.isIgnoreError(err) && _this.limit > 1) {
        _this.next(false, item);
      } else {
        item.deferred.reject(err);
        _this.next();
      }
    });
  };
  /**
   * run task
   */


  ConcurrentTaskLimit.prototype.run = function run(fn) {
    var deferred = (0, _stcHelper.defer)();
    this.deferreds.push({
      deferred: deferred,
      fn: fn
    });
    this._runTask();
    return deferred.promise;
  };

  return ConcurrentTaskLimit;
}();

exports.default = ConcurrentTaskLimit;
//# sourceMappingURL=concurrent_limit.js.map