'use strict';

exports.__esModule = true;

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _stringify = require('babel-runtime/core-js/json/stringify');

var _stringify2 = _interopRequireDefault(_stringify);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _stcHelper = require('stc-helper');

var _cluster = require('cluster');

var _stcPluginInvoke = require('stc-plugin-invoke');

var _stcPluginInvoke2 = _interopRequireDefault(_stcPluginInvoke);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _url = require('url');

var _url2 = _interopRequireDefault(_url);

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _helper = require('./helper.js');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var homePath = process.env[process.platform === 'win32' ? 'USERPROFILE' : 'HOME'];

/**
 * stc plugin abstract class
 */

var StcPlugin = function () {
  /**
   * constructor
   */

  function StcPlugin(file) {
    var opts = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];
    (0, _classCallCheck3.default)(this, StcPlugin);

    this.file = file;
    this.options = opts.options || {};
    this.stc = opts.stc;
    this.config = this.stc.config;
    this.TokenType = this.stc.flkit.TokenType;
    this.include = opts.include;
    //can not use ext in sub plugins
    this._ext = opts.ext || {};
    //store other properties
    this._prop = {};
  }
  /**
   * get matches
   */


  /**
   * get or set properties
   */

  StcPlugin.prototype.prop = function prop(name, value) {
    if (value === undefined) {
      return this._prop[name];
    }
    this._prop[name] = value;
    return this;
  };
  /**
   * is template file
   */


  StcPlugin.prototype.isTpl = function isTpl() {
    return !!this.file.prop('tpl');
  };
  /**
   * get md5 value of plugin & options
   */


  StcPlugin.prototype.getMd5 = function getMd5() {
    if (this.prop('md5')) {
      return this.prop('md5');
    }
    var value = (0, _stcHelper.md5)(this.constructor.toString() + (0, _stringify2.default)(this.options));
    this.prop('md5', value);
    return value;
  };
  /**
   * get file content
   */


  StcPlugin.prototype.getContent = function () {
    var _ref = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee() {
      var encoding = arguments.length <= 0 || arguments[0] === undefined ? null : arguments[0];
      var content;
      return _regenerator2.default.wrap(function _callee$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              if (!_cluster.isMaster) {
                _context.next = 2;
                break;
              }

              return _context.abrupt('return', this.file.getContent(encoding));

            case 2:
              _context.next = 4;
              return this.stc.cluster.workerInvoke({
                method: 'getContent',
                encoding: encoding,
                file: this.file.path
              });

            case 4:
              content = _context.sent;

              if (!(encoding !== null)) {
                _context.next = 7;
                break;
              }

              return _context.abrupt('return', content);

            case 7:
              return _context.abrupt('return', new Buffer(content, 'base64'));

            case 8:
            case 'end':
              return _context.stop();
          }
        }
      }, _callee, this);
    }));

    function getContent(_x2) {
      return _ref.apply(this, arguments);
    }

    return getContent;
  }();
  /**
   * set file content
   */


  StcPlugin.prototype.setContent = function setContent(content) {
    (0, _helper.checkInMaster)('setContent');
    (0, _helper.checkRunIsExecute)(this, 'setContent');
    this.file.setContent(content);
    return this;
  };
  /**
   * get file ast
   */


  StcPlugin.prototype.getAst = function () {
    var _ref2 = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee4() {
      var _this = this;

      var content;
      return _regenerator2.default.wrap(function _callee4$(_context4) {
        while (1) {
          switch (_context4.prev = _context4.next) {
            case 0:
              _context4.next = 2;
              return this.getContent('utf8');

            case 2:
              content = _context4.sent;

              if (!_cluster.isMaster) {
                _context4.next = 7;
                break;
              }

              if (!this.file.hasAst()) {
                _context4.next = 6;
                break;
              }

              return _context4.abrupt('return', this.file.getAst());

            case 6:
              return _context4.abrupt('return', (0, _helper.getAst)(this, content, (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee2() {
                var ret;
                return _regenerator2.default.wrap(function _callee2$(_context2) {
                  while (1) {
                    switch (_context2.prev = _context2.next) {
                      case 0:
                        if (!(_this.config.cluster === false)) {
                          _context2.next = 2;
                          break;
                        }

                        return _context2.abrupt('return', _this.file.getAst());

                      case 2:
                        _context2.next = 4;
                        return _this.stc.cluster.masterInvoke({
                          type: 'getAst',
                          content: content,
                          file: _this.file.path
                        });

                      case 4:
                        ret = _context2.sent;

                        _this.file.setAst(ret);
                        return _context2.abrupt('return', ret);

                      case 7:
                      case 'end':
                        return _context2.stop();
                    }
                  }
                }, _callee2, _this);
              }))));

            case 7:
              return _context4.abrupt('return', (0, _helper.getAst)(this, content, (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee3() {
                var ast, ret;
                return _regenerator2.default.wrap(function _callee3$(_context3) {
                  while (1) {
                    switch (_context3.prev = _context3.next) {
                      case 0:
                        _context3.next = 2;
                        return _this.stc.cluster.workerInvoke({
                          method: 'getAstIfExist',
                          file: _this.file.path
                        });

                      case 2:
                        ast = _context3.sent;

                        if (!ast) {
                          _context3.next = 5;
                          break;
                        }

                        return _context3.abrupt('return', ast);

                      case 5:
                        _this.file.setContent(content);
                        _context3.next = 8;
                        return _this.file.getAst();

                      case 8:
                        ret = _context3.sent;
                        _context3.next = 11;
                        return _this.stc.cluster.workerInvoke({
                          method: 'updateAst',
                          file: _this.file.path,
                          ast: ret
                        });

                      case 11:
                        return _context3.abrupt('return', ret);

                      case 12:
                      case 'end':
                        return _context3.stop();
                    }
                  }
                }, _callee3, _this);
              }))));

            case 8:
            case 'end':
              return _context4.stop();
          }
        }
      }, _callee4, this);
    }));

    function getAst() {
      return _ref2.apply(this, arguments);
    }

    return getAst;
  }();
  /**
   * set ast
   */


  StcPlugin.prototype.setAst = function setAst(ast) {
    (0, _helper.checkInMaster)('setAst');
    (0, _helper.checkRunIsExecute)(this, 'setAst');
    this.file.setAst(ast);
    return this;
  };
  /**
   * add file dependence
   */


  StcPlugin.prototype.addDependence = function addDependence(dependencies) {
    var _this2 = this;

    if (!_cluster.isMaster) {
      throw new Error('addDependence must be invoked in master');
    }
    if (!(0, _stcHelper.isArray)(dependencies)) {
      dependencies = [dependencies];
    }
    dependencies = dependencies.map(function (item) {
      if (!(0, _stcHelper.isString)(item)) {
        return item;
      }
      var filepath = _this2.getResolvePath(item);
      var file = _this2.stc.resource.getFileByPath(filepath, _this2.file.path);
      if (!file) {
        throw new Error('file ' + item + ' is not exist in ' + _this2.file.path);
      }
      return file;
    });
    this.file.dependence.add(dependencies);
    return dependencies;
  };
  /**
   * get file dependence
   */


  StcPlugin.prototype.getDependence = function getDependence() {
    var file = arguments.length <= 0 || arguments[0] === undefined ? this.file : arguments[0];

    return file.dependence.get();
  };

  /**
   * add file
   */


  StcPlugin.prototype.addFile = function () {
    var _ref5 = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee5(filepath, content, virtual) {
      var resolvePath;
      return _regenerator2.default.wrap(function _callee5$(_context5) {
        while (1) {
          switch (_context5.prev = _context5.next) {
            case 0:
              resolvePath = this.getResolvePath(filepath);

              if (!_cluster.isMaster) {
                _context5.next = 3;
                break;
              }

              return _context5.abrupt('return', this.stc.resource.addFile(resolvePath, content, virtual));

            case 3:
              _context5.next = 5;
              return this.stc.cluster.workerInvoke({
                method: 'addFile',
                file: resolvePath,
                virtual: virtual,
                content: content
              });

            case 5:
              return _context5.abrupt('return', this.stc.resource.createFile(resolvePath, content, virtual));

            case 6:
            case 'end':
              return _context5.stop();
          }
        }
      }, _callee5, this);
    }));

    function addFile(_x5, _x6, _x7) {
      return _ref5.apply(this, arguments);
    }

    return addFile;
  }();
  /**
   * get resolve path
   */


  StcPlugin.prototype.getResolvePath = function getResolvePath(filepath) {
    // parse filepath, remove query & hash in filepath
    filepath = decodeURIComponent(_url2.default.parse(filepath).pathname);
    var currentFilePath = _path2.default.dirname(this.file.path);
    var resolvePath = _path2.default.resolve(currentFilePath, filepath);
    var currentPath = process.cwd() + _path2.default.sep;
    if (resolvePath.indexOf(currentPath) === 0) {
      resolvePath = resolvePath.slice(currentPath.length);
    }
    return resolvePath;
  };
  /**
   * get file by path
   */


  StcPlugin.prototype.getFileByPath = function getFileByPath(filepath) {
    filepath = this.getResolvePath(filepath);
    if (_cluster.isMaster) {
      return this.stc.resource.getFileByPath(filepath, this.file.path);
    }
    return this.stc.resource.createFile(filepath);
  };

  /**
   * invoke self plugin
   */


  StcPlugin.prototype.invokeSelf = function () {
    var _ref6 = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee6() {
      var file = arguments.length <= 0 || arguments[0] === undefined ? this.file : arguments[0];
      var props = arguments[1];
      return _regenerator2.default.wrap(function _callee6$(_context6) {
        while (1) {
          switch (_context6.prev = _context6.next) {
            case 0:
              if ((0, _stcHelper.isString)(file)) {
                file = this.getFileByPath(file);
              }
              return _context6.abrupt('return', this.invokePlugin(this.constructor, file, props));

            case 2:
            case 'end':
              return _context6.stop();
          }
        }
      }, _callee6, this);
    }));

    function invokeSelf(_x8, _x9) {
      return _ref6.apply(this, arguments);
    }

    return invokeSelf;
  }();

  /**
   * invoke plugin
   */


  StcPlugin.prototype.invokePlugin = function () {
    var _ref7 = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee7(plugin) {
      var file = arguments.length <= 1 || arguments[1] === undefined ? this.file : arguments[1];
      var props = arguments.length <= 2 || arguments[2] === undefined ? {} : arguments[2];
      var instance, name, data;
      return _regenerator2.default.wrap(function _callee7$(_context7) {
        while (1) {
          switch (_context7.prev = _context7.next) {
            case 0:
              if ((0, _stcHelper.isString)(file)) {
                file = this.getFileByPath(file);
              }
              instance = new _stcPluginInvoke2.default(plugin, file, {
                stc: this.stc,
                options: this.options,
                ext: plugin === this.constructor ? this._ext : {}
              });
              //set prop for plugin

              for (name in props) {
                instance.pluginInstance.prop(name, props[name]);
              }

              if (!_cluster.isMaster) {
                _context7.next = 5;
                break;
              }

              return _context7.abrupt('return', instance.run());

            case 5:
              _context7.next = 7;
              return instance.run();

            case 7:
              data = _context7.sent;
              _context7.next = 10;
              return this.stc.cluster.workerInvoke({
                method: 'update',
                file: this.file.path,
                data: data
              });

            case 10:
              return _context7.abrupt('return', data);

            case 11:
            case 'end':
              return _context7.stop();
          }
        }
      }, _callee7, this);
    }));

    function invokePlugin(_x11, _x12, _x13) {
      return _ref7.apply(this, arguments);
    }

    return invokePlugin;
  }();

  /**
   * async content replace
   * must be use RegExp
   */


  StcPlugin.prototype.asyncReplace = function asyncReplace() {
    var content = arguments.length <= 0 || arguments[0] === undefined ? '' : arguments[0];
    var replace = arguments[1];
    var callback = arguments[2];

    return (0, _stcHelper.asyncReplace)(content, replace, callback);
  };
  /**
   * get or set cache
   */


  StcPlugin.prototype.cache = function () {
    var _ref8 = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee8(name, value) {
      var isFn, instance, ret, awaitInstance, ret2, _ret, _awaitInstance, _ret2;

      return _regenerator2.default.wrap(function _callee8$(_context8) {
        while (1) {
          switch (_context8.prev = _context8.next) {
            case 0:
              isFn = typeof value === 'function';

              if (!_cluster.isMaster) {
                _context8.next = 22;
                break;
              }

              instance = (0, _helper.getCacheInstance)(this);

              if (!(value === undefined)) {
                _context8.next = 5;
                break;
              }

              return _context8.abrupt('return', instance.get(name));

            case 5:
              if (!isFn) {
                _context8.next = 19;
                break;
              }

              _context8.next = 8;
              return instance.get(name);

            case 8:
              ret = _context8.sent;

              if (!(ret !== undefined)) {
                _context8.next = 11;
                break;
              }

              return _context8.abrupt('return', ret);

            case 11:
              awaitInstance = (0, _helper.getAwaitInstance)(this.getMd5());
              _context8.next = 14;
              return awaitInstance.run(name, value);

            case 14:
              ret2 = _context8.sent;

              if (!(ret2 !== undefined)) {
                _context8.next = 18;
                break;
              }

              _context8.next = 18;
              return instance.set(name, ret2);

            case 18:
              return _context8.abrupt('return', ret2);

            case 19:
              _context8.next = 21;
              return instance.set(name, value);

            case 21:
              return _context8.abrupt('return', this);

            case 22:
              if (!isFn) {
                _context8.next = 36;
                break;
              }

              _context8.next = 25;
              return this.stc.cluster.workerInvoke({
                method: 'cache',
                key: this.getMd5(),
                name: name
              });

            case 25:
              _ret = _context8.sent;

              if (!(_ret !== undefined)) {
                _context8.next = 28;
                break;
              }

              return _context8.abrupt('return', _ret);

            case 28:
              _awaitInstance = (0, _helper.getAwaitInstance)(this.getMd5());
              _context8.next = 31;
              return _awaitInstance.run(name, value);

            case 31:
              _ret2 = _context8.sent;

              if (!(_ret2 !== undefined)) {
                _context8.next = 35;
                break;
              }

              _context8.next = 35;
              return this.stc.cluster.workerInvoke({
                method: 'cache',
                key: this.getMd5(),
                name: name,
                value: _ret2
              });

            case 35:
              return _context8.abrupt('return', _ret2);

            case 36:
              return _context8.abrupt('return', this.stc.cluster.workerInvoke({
                method: 'cache',
                key: this.getMd5(),
                name: name,
                value: value
              }));

            case 37:
            case 'end':
              return _context8.stop();
          }
        }
      }, _callee8, this);
    }));

    function cache(_x17, _x18) {
      return _ref8.apply(this, arguments);
    }

    return cache;
  }();
  /**
   * storage
   */


  StcPlugin.prototype.storage = function storage(name, value) {
    var savePath = _path2.default.normalize(homePath + '/.stc/storage/' + name + '.json');
    (0, _stcHelper.mkdir)(_path2.default.dirname(savePath));
    if (value === undefined) {
      if ((0, _stcHelper.isFile)(savePath)) {
        var data = _fs2.default.readFileSync(savePath, 'utf8');
        return JSON.parse(data);
      }
      return;
    }
    _fs2.default.writeFileSync(savePath, (0, _stringify2.default)(value));
  };
  /**
   * create token
   */


  StcPlugin.prototype.createToken = function createToken(type, value, referToken) {
    return this.stc.flkit.createToken(type, value, referToken);
  };
  /**
   * create raw token in html
   */


  StcPlugin.prototype.createRawToken = function createRawToken(type, value, referToken) {
    return this.stc.flkit.createRawToken(type, value, referToken);
  };
  /**
   * concurrent limit task
   */


  StcPlugin.prototype.concurrentLimit = function concurrentLimit(fn, ignoreErrorFn, limit) {
    var key = arguments.length <= 3 || arguments[3] === undefined ? this.constructor.name : arguments[3];

    var instance = (0, _helper.getConcurrentLimitInstance)(limit, ignoreErrorFn, key);
    return instance.run(fn);
  };
  /**
   * await
   */


  StcPlugin.prototype.await = function await(key, fn) {
    var instance = (0, _helper.getAwaitInstance)(key);
    return instance.run(key, fn);
  };
  /**
   * throw fatal error
   */


  StcPlugin.prototype.fatal = function fatal(message, line, column) {
    var file = arguments.length <= 3 || arguments[3] === undefined ? this.file.path : arguments[3];

    var msg = {
      message: message,
      className: this.constructor.name,
      file: file,
      line: line,
      column: column
    };
    throw new Error((0, _stringify2.default)(msg));
  };

  /**
   * show error log
   */


  StcPlugin.prototype.error = function error(message, line, column) {
    var file = arguments.length <= 3 || arguments[3] === undefined ? this.file.path : arguments[3];

    (0, _helper.checkRunIsExecute)(this, 'error');
    this.stc.log.error({
      message: message,
      line: line,
      column: column,
      file: file,
      className: this.constructor.name
    });
  };

  /**
  * show warning log
  */


  StcPlugin.prototype.warning = function warning(message, line, column) {
    var file = arguments.length <= 3 || arguments[3] === undefined ? this.file.path : arguments[3];

    (0, _helper.checkRunIsExecute)(this, 'warning');
    this.stc.log.warning({
      message: message,
      line: line,
      column: column,
      file: file,
      className: this.constructor.name
    });
  };

  /**
  * show notice log
  */


  StcPlugin.prototype.notice = function notice(message, line, column) {
    var file = arguments.length <= 3 || arguments[3] === undefined ? this.file.path : arguments[3];

    (0, _helper.checkRunIsExecute)(this, 'notice');
    this.stc.log.notice({
      message: message,
      line: line,
      column: column,
      file: file,
      className: this.constructor.name
    });
  };

  /**
   * run
   */


  StcPlugin.prototype.run = function run() {};
  /**
   * update
   */


  StcPlugin.prototype.update = function update() {};

  (0, _createClass3.default)(StcPlugin, [{
    key: 'matches',
    get: function get() {
      return this.stc.resource.match(this.file, this.include);
    }
  }]);
  return StcPlugin;
}();

exports.default = StcPlugin;
//# sourceMappingURL=index.js.map